#ifndef __MATRIX_4X4_H_
#define __MATRIX_4X4_H_

#include "Vec3.h"

namespace ambenv {

  const unsigned M_DIM = 4; 

  class Matrix4x4 {
  public:
    Matrix4x4();
    Matrix4x4( const Matrix4x4 & rhs ); 
    
    void zero(); //!< set all entries to zero
    void identity(); //!< set the matrix to the identity

    bool operator==( const Matrix4x4 & rhs ) const; //!< point-wise equality test
    bool operator!=( const Matrix4x4 & rhs ) const; //!< point-wise inequality test
    
    Matrix4x4 operator*( const Matrix4x4 & rhs ) const; //!< matrix multiplication
    Matrix4x4 operator*( Real value ) const; //!< scalar muliplication
    Matrix4x4 operator/( Real value ) const; //!< scalar division
    Matrix4x4 operator+( const Matrix4x4 & rhs ) const; //!< point-wise matrix addition
    Matrix4x4 operator-( const Matrix4x4 & rhs ) const; //!< point-wise matrix subtraction
    
    void operator*=( const Matrix4x4 & rhs ); //!< matrix multiplication
    void operator*=( Real value ); //!< scalar multiplication
    void operator/=( Real value ); //!< scalar division
    void operator+=( const Matrix4x4 & rhs );	//!< point-wise matrix addition
    void operator-=( const Matrix4x4 & rhs );	//!< point-wise matrix subtraction
  
    Real operator()( unsigned i, unsigned j ) const; //!< return element in i-th row, j-th column
    void operator()( unsigned i, unsigned j, Real value ); //!< set i-th row, j-th column element
    
    Vec3 multVec( const Vec3 & rhs, bool homogeneous = true ) const; //!< multiply the vector by this matrix

    Matrix4x4 transpose() const; //!< return the transpose of this matrix
    bool invert(); //!< invert this matrix in place, return false if it is not invertible

  private:
    Real mp_data[M_DIM*M_DIM];
  };
  
}; /* namespace ambenv */

#endif /* __MATRIX_4X4_H_ */
