/*
*	Linear equation solution by Gauss-Jordan elimination. a[1..n][1..n] is
*	an input matrix of n by n elements.  b[1..n][1..m] is an input matrix
*	of size n by m containing the m right hand side vectors.  On output,
*	a is replaced by its matrix inverse, and b is replaced by the corresponding
*	set of solution vectors.
*
*	m should be 1 when there is one b vector.
*/
#include "gaussj.h"

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define SWAP(a,b) {float temp=(a);(a)=(b);(b)=temp;}

int * ivector( int m, int n )
{
  int * vec = (int *)malloc( (n+1)*sizeof( int ) );
  memset( vec, 0, (n+1)*sizeof( int ) );
  return( vec );
}

void free_ivector( int * iv, int m, int n )
{
  if( iv ) {
    free( iv );
    iv = 0;
  }
}

int gaussj(a,n,b,m)
     float **a,**b;
     int n,m;
{
  int *indxc,*indxr,*ipiv;
  int i,icol,irow,j,k,l,ll; //,*ivector();
  float big,dum,pivinv;
  //void nrerror(),free_ivector();
  
  indxc=ivector(1,n);
  indxr=ivector(1,n);
  ipiv=ivector(1,n);

  for (j=1;j<=n;j++) ipiv[j]=0;
  for (i=1;i<=n;i++) {
    big=0.0;
    for (j=1;j<=n;j++)
      if (ipiv[j] != 1)
	for (k=1;k<=n;k++) {
	  if (ipiv[k] == 0) {
	    if (fabs(a[j][k]) >= big) {
	      big=(float)fabs(a[j][k]);
	      irow=j;
	      icol=k;
	    }
	  } else if (ipiv[k] > 1) { 
	    // nrerror("GAUSSJ: Singular Matrix-1");
	    free_ivector(ipiv,1,n);
	    free_ivector(indxr,1,n);
	    free_ivector(indxc,1,n);
	    return( 0 );
	  }
	}
    ++(ipiv[icol]);
    if (irow != icol) {
      for (l=1;l<=n;l++) SWAP(a[irow][l],a[icol][l])
			   for (l=1;l<=m;l++) SWAP(b[irow][l],b[icol][l])
						}
    indxr[i]=irow;
    indxc[i]=icol;
    if (a[icol][icol] == 0.0) {
      // nrerror("GAUSSJ: Singular Matrix-2");
      free_ivector(ipiv,1,n);
      free_ivector(indxr,1,n);
      free_ivector(indxc,1,n);
      return( 0 );
    }
    pivinv=1.0f/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=1;l<=n;l++) a[icol][l] *= pivinv;
    for (l=1;l<=m;l++) b[icol][l] *= pivinv;
    for (ll=1;ll<=n;ll++)
      if (ll != icol) {
	dum=a[ll][icol];
	a[ll][icol]=0.0;
	for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
	for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
      }
  }
  for (l=n;l>=1;l--) {
    if (indxr[l] != indxc[l])
      for (k=1;k<=n;k++)
	SWAP(a[k][indxr[l]],a[k][indxc[l]]);
  }

  free_ivector(ipiv,1,n);
  free_ivector(indxr,1,n);
  free_ivector(indxc,1,n);

  return( 1 );
}

#undef SWAP
